@@ -7,13 +7,13 @@ import struct
 import sys
 import urllib2
 
-_RELEASE_URI = "https://releases.mozilla.com"
-_B2G_LATEST = _RELEASE_URI + "/b2g/latest"
-_CHUNK_SIZE = 32768 
+_RELEASE_URI = "https://releases.mozilla.com/b2g/"
+_B2G_LATEST = "latest"
+_CHUNK_SIZE = 32768
 
-def find_url(user, password, device):
+def find_url(user, password, keys, date=None):
     """
-    Returns the url of the build specified by device.
+    Returns the url of the build specified by keys or None.
     """
     # install http basic auth opener
     auth_handler = urllib2.HTTPBasicAuthHandler()
@@ -24,14 +24,19 @@ def find_url(user, password, device):
     opener = urllib2.build_opener(auth_handler)
     urllib2.install_opener(opener)
 
-    data = urllib2.urlopen(_B2G_LATEST)
+    if not date:
+        uri = _RELEASE_URI + _B2G_LATEST
+    else:
+        uri = _RELEASE_URI + date
+
+    data = urllib2.urlopen(uri)
     soup = BeautifulSoup(data.read())
-    # get trunk build if it exists 
+    # get trunk build if it exists
     try:
-        link = soup.find_all('a', href=lambda x: device in x and 'trunk' in x)[0]
+        link = soup.find_all('a', href=lambda x: len(keys) == len([k for k in keys if k in x]))[0]
     except:
-        link = soup.find_all('a', href=lambda x: device in x)[0]
-    return _B2G_LATEST + '/' + link['href']
+        return None
+    return uri + '/' + link['href']
 
 def _chunk_report(url, bytes_so_far, total_size):
     percent = float(bytes_so_far) / total_size
@@ -42,69 +47,70 @@ def _chunk_report(url, bytes_so_far, total_size):
         sys.stdout.write('\n')
     sys.stdout.flush()
 
-def save_as(user, password, device, savepath, silent=False):
+def save_as(user, password, keys, savepath, date=None, silent=False):
     """
-    Finds the url of the build specified by device 
+    Finds the url of the build specified by device
     and saves it to savepath.
+
+    If no builds were found, returns None
     """
-    url = find_url(user, password, device)
-    
+    url = find_url(user, password, keys, date=date)
+    if url is None:
+        if not silent:
+            print "Could not find build matching [%s]" % ", ".join(keys)
+        return None
+
     response = urllib2.urlopen(url)
     total_size = int(response.info().getheader('Content-Length').strip())
     bytes_so_far = 0
-    
+
     savepath = os.path.realpath(savepath)
     outfile = open(savepath, "wb")
     while True:
         chunk = response.read(_CHUNK_SIZE)
         if not chunk:
              break
-        
+
         bytes_so_far += len(chunk)
         outfile.write(chunk)
 
         if not silent:
             _chunk_report(url, bytes_so_far, total_size)
-    
+
     outfile.close()
     return savepath
 
 def cli(args=sys.argv[1:]):
-    valid_devices = ['emulator-arm',
-                     'emulator-x86', 
-                     'nexus-s',
-                     'otoro',
-                     'sgs2']
-
     parser = optparse.OptionParser(usage="%prog [options]")
-    parser.add_option("--user", dest="user",
+    parser.add_option("-u", "--user", dest="user",
                       action="store", default=getpass.getuser(),
                       help="Username for the b2g nightly releases")
-    parser.add_option("--password", dest="password",
-                      action="store", default=None,
-                      help="Password for the b2g nightly releases")
-    parser.add_option("--device", dest="device",
-                      action="store", default=None,
-                      help="Device for which to get build. "
-                           "One of %s" % ", ".join(valid_devices))
+    parser.add_option("--key", dest="keys",
+                      action="append", default=None,
+                      help="List of keywords to use for finding which "
+                           " build to get. First build which matches all is chosen")
     parser.add_option("-o", "--output-file", dest="outfile",
                       action="store", default=None,
                       help="File path to save the build. If not specified, "
-                           "the URL will be printed to stdout.")
+                           "the URL will be printed to stdout")
+    parser.add_option("--date", dest="date",
+                      action="store", default=None,
+                      help="Date of the nightly to download in the format: "
+                           "YYYY-MM-DD")
 
     opt, arguments = parser.parse_args(args)
-   
+
     # verify options
-    if not opt.device in valid_devices:
-        print "Specify a valid device with --device: %s" % ", ".join(valid_devices)
+    if not opt.keys:
+        parser.error("Must specify at least one --key")
         return 1
-    if not opt.password:
-        opt.password = getpass.getpass("Password for %s:" % opt.user)
+
+    password = getpass.getpass("Password for %s:" % opt.user)
 
     if not opt.outfile:
-        print find_url(opt.user, opt.password, opt.device)
+        print find_url(opt.user, password, opt.keys, date=opt.date)
     else:
-        print save_as(opt.user, opt.password, opt.device, opt.outfile)
+        print save_as(opt.user, password, opt.keys, opt.outfile, date=opt.date)
 
 
 if __name__ == '__main__':
